
% multiinputtool

% This is the program in the Input-Signal-Creation Suite that obtains sets of input signal periods for multiinput system identification

% The program requires one input from the program createsignal

  maxprime=2281;  % maxprime defines the largest Galois field GF(maxprime) used in the program createsignal

% The program uses fieldsetsx for x=1,2,3,4,5 and 6
% Each field GF(fieldsetx(index))is a prime field of the form GF(2*k*x+1)

% The program uses periodsets defined by periodsetx=fieldsetx.*2x+2x for x=1,2,3,4,5,6

% periodset1=fieldset1.*2+2   is for Class 1 signals with 3 levels and periods multiples of 4
% periodset2=fieldset2.*4+4   is for Class 1 signals with 3 or 5 levels and periods multiples of 8
% periodset3=fieldset3.*6+6   is for Class 1 signals with 5 or 7 levels or Class 2 signals with 3, 5 or 7 levels and periods multiples of 10
% periodset4=fieldset4.*8+8   is for Class 1 signals with 3, 5 or 7 levels and periods multiples of 16
% periodset5=fieldset5.*10+10 is for Class 1 signals with 5 or 7 levels and periods multiples of 20
% periodset6=fieldset6.*12+12 is for Class 1 signals with 3, 5 or 7 levels or Class 2 signals with 3, 5 or 7 levels and periods multiples of 24

% The program also generates but at present does not use fieldset7 in which each field GF(fieldset7(index))is a prime field of the form GF(4*k-1)

% The program also generates periodsetsx for x=7,8,9,10 defined as follows but only periodset10 is used at present 

% periodset7=fieldset7        is for Class 0 signals with 2 levels and periods primes
% periodset8=fieldset1        is for Class 0 signals with 3 levels and periods primes
% periodset9=fieldset7.*2     is for Class 1 signals with 2 levels and periods 2*primes
% periodset10=fieldset1.*2    is for Class 1 signals with 3 levels and periods 2*primes

% Generate the fieldsets and periodsets

[fieldset1 fieldset2 fieldset3 fieldset4 fieldset5 fieldset6 fieldset7 periodset1 periodset2 periodset3 periodset4 periodset5 periodset6 periodset7 periodset8 periodset9 periodset10]=getsets(maxprime);

% Open the user interface dialogue and display the program function and signal properties

disp(' ')
disp('Welcome to the Multiinput Support Tool in the Input-Signal-Creation Program Suite')
disp(' ')
disp('Use this tool to determine the periods of input signals suitable for multiinput system identification')
disp(' ')
disp('Two classes of input signal can be used for multiinput system identification')
disp(' ')
disp('  Class 1 signals for which the power of all harmonics that are multiples of 2 is zero')
disp('  Class 2 signals for which the power of all harmonics that are multiples of 2 and 3 is zero')
disp(' ')

% Select the signal class

disp('Select the class of input signal from the menu')
menuout=menu('Signal Class','Class 1','Class 2');          % display menu
class=menuout;                                           % get class from menu out
disp(' ')
disp(['  The signals are Class ',int2str(class) ,''])    % display selected class
disp(' ')
 
% Select the number of signal levels

if  class==1
    disp('Select the number of input signal levels from the menu')
    menuout=menu('Signal Levels','3 Levels','5 Levels','7 Levels'); % display menu for Class 1 signals
    levels=2*menuout+1;                                 % get levels from menuout for remaining Class 1 signals
    disp(' ')
    disp(['  The signals have ',int2str(levels) ,' levels']) % display selected levels
    disp(' ')
end

% Select the number of system inputs

disp('Select the number of system inputs from the menu')
if      class==1&&levels==3
        menuout=menu('System Inputs','2 Inputs','3 Inputs','4 Inputs','5 Inputs'); % display menu for Class 1 3-level signals
else    menuout=menu('System Inputs','2 Inputs','3 Inputs'); % display menu for Class 1 5 or 7-level signals
end
inputs=menuout+1;                                       % get inputs from menuout
disp(' ')
disp(['  The system has ',int2str(inputs) ,' inputs'])  % display selected inputs
disp(' ')

% Generate the merged periodsets as periodsetsmerged

if      class==1&&levels==3
        periodsetsmerged=unique(sort([periodset1 periodset2 periodset4 periodset6 periodset10])); % arrange periods in ascending order
elseif  class==1&&levels==5
        periodsetsmerged=unique(sort([periodset2 periodset3 periodset4 periodset5 periodset6])); % arrange periods in ascending order
elseif  class==1&&levels==7
        periodsetsmerged=unique(sort([periodset3 periodset4 periodset5 periodset6])); % arrange periods in ascending order
elseif  class==2
        periodsetsmerged=unique(sort([periodset3 periodset6])); % arrange selection in ascending order
end

% Generate the sets of periods
                                                        
multiperiods=zeros(inputs,length(periodsetsmerged));
j=1;                                                    % index of first result
for i=1:length(periodsetsmerged);                       % whole range of periodsetsmerged    
    k=1;                                                % first multiple of 2
    while   k<inputs
            index=find(periodsetsmerged==2^k*periodsetsmerged(i)); % get index of multiple of 2    
            if      ~isempty(index)                     % if multiple exists
                    k=k+1;                              % continue until all multiples used
                    if  k==inputs                       % if last multiple 
                        multiperiods(1,j)=periodsetsmerged(index); % enter in results
                        j=j+1;                          % increment results index
                    end      
            else    k=inputs;                           % terminate this case  
            end
    end
end

% Display the sets of periods

number=nnz(multiperiods(1,:));                          % number of sets of periods
multiperiods=multiperiods(:,1:number);                  % remove redundant zeros
for row=2:inputs                                        % number of inputs
    multiperiods(row,:)=multiperiods(1,:)./2^(row-1);   % remaining rows of output
end
if      class==1
        disp(['There are ',int2str(number) ,' sets of periods of Class ',int2str(class) ,' ',int2str(levels), '-level signals suitable for identifying a ',int2str(inputs),'-input system'])
        disp(' ')
        disp(int2str(multiperiods))
        disp(' ')
        if      levels==3
                disp('The signals in these sets may be replaced by 2-, 5- or 7-level signals with the same periods')
        elseif  levels==5
                disp('The signals in these sets may be replaced by 3- or 7-level signals with the same periods')
        elseif  levels==7
                disp('The signals in these sets may be replaced by 3- or 5-level signals with the same periods')
        end
else
        disp(['There are ',int2str(number) ,' sets of periods of Class 2 signals suitable for identifying a ',int2str(inputs),'-input system'])
        disp(' ')
        disp(int2str(multiperiods))
        disp(' ')
        disp('The signals in these sets may have 3, 5 or 7 levels')
end
